-----------------------------------------------------------------------------
--
-- data
--  \threat
--    \<playerName>
--      - amount
--		\targetDetails
--        <targetHash> : amount
--
-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AssessmentThreat")
L:RegisterTranslations("enUS", function() return {
	["Threat List"] = true,
	["Threat Deficit"] = true,
	["Aggro Gain"] = true,
	["Aggro Gain Color"] = true,
	["Threat"] = true,
	["TPS"] = true,
	["Threat gap before Aggro gain on target"] = true,
} end)

local AssessmentThreat = Assessment:NewModule("Threat", "AceEvent-2.0")
local Threat = AceLibrary("Threat-1.0")
local THREAT_GLOBAL_TARGET = Threat.GlobalTarget

-----------------------------------------------------------------------------

local aggroGainName = L["Aggro Gain"]

-----------------------------------------------------------------------------

Assessment.TagData.targetthreat =
{
	method = function(data)
				local targetName = UnitName("target")
				if not targetName then return 0 end
				if data.key == aggroGainName then
					return Assessment.TagData.aggrogaintargetthreat.method(data)
				else
					return Threat:GetThreat(data.key, targetName)
				end
			 end,
	data			= { "target", "threat" },
	guaranteeResult = true,
	numeric			= true
}

Assessment.TagData.maxtargetthreat =
{
	method = function(data)
				local targetName = UnitName("target")
				if not targetName then return 0 end
				return Threat:GetMaxThreatOnTarget(targetName)
			 end,
	data			= { "target", "threat" },
	guaranteeResult = true,
	numeric			= true
}

Assessment.TagData.aggrogaintargetthreat =
{
	method = function(data)
				local targetName = UnitName("target")
				if not targetName then return 0 end

				if UnitIsUnit("player", "targettarget") then
					return Threat:GetThreat(UnitName("player"), targetName)
				end
				
				local multiplier = Threat:UnitInMeleeRange("target") and 1.1 or 1.3
				local targetTargetName = UnitName("targettarget")
				if not targetTargetName then
					return Threat:GetMaxThreatOnTarget(targetName)
				end
				return Threat:GetThreat(targetTargetName, targetName) * multiplier
			 end,
	data			= { "target", "threat" },
	guaranteeResult = true,
	numeric			= true
}

Assessment.TagData.threat =
{
	method = function(data)
				if data.key == aggroGainName then return 0 end

				local fightData = data.fightData
				if not fightData then return 0 end
				
				local threat = fightData.threat
				if not threat then return 0 end
				
				if not threat[data.key] then return 0 end
				return threat[data.key].amount					
			 end,
	data			= { "threat" },
	guaranteeResult = true,
	numeric			= true
}

Assessment.TagData.maxthreat =
{
	method = function(data)
				local fightData = data.fightData
				if not fightData then return 0 end
				
				local threat = fightData.threat
				if not threat then return 0 end

				local maxThreat = 0
				for k,v in pairs(threat) do
					if v.amount > maxThreat then
						maxThreat = v.amount
					end
				end
				return maxThreat
			 end,
	data			= { "threat" },
	guaranteeResult = true,
	numeric			= true
}

Assessment.TagData.threattargetname =
{
	method = function(data) 
		if UnitExists("target") and not UnitIsFriend("player", "target") then
			return UnitName("target")
		end
	end,
}

Assessment.TagData.isusingforeignthreatsource =
{
	method = function(data) return Threat:IsUsingForeignThreatSource(data.key) end,
}

-----------------------------------------------------------------------------

local function ThreatListIterator(state, index)
	if not index then
		return aggroGainName
	elseif index == aggroGainName then
		return next(state)
	else
		return next(state, index)
	end
end

Assessment.ListType.ThreatList =
{
    method = function(fightData)
                if not fightData.threat then return function() return nil end end
				if UnitExists("targettarget") and not UnitIsFriend("player", "target") and not UnitIsUnit("player", "targettarget") then
					return ThreatListIterator, fightData.threat, nil
				else
					return pairs(fightData.threat)
				end
             end,
    data = "threat",
    description = L["Threat List"],
}

-----------------------------------------------------------------------------

local aggroGainColor = { 1, 0, 0 }
Assessment.ColorType.AggroGain =
{
	method = function(key)
				if key == aggroGainName then
					return aggroGainColor
				end
			 end,
	description = L["Aggro Gain Color"]
}

-----------------------------------------------------------------------------

Assessment.DefaultDisplaySets.Threat =
{
	titleTag		= "[threatTargetName:yellow][~threatTargetName:\"Threat\"]",
	listType		= "ThreatList", 
	barValueTag		= "select(threatTargetName, targetthreat, threat)", 
	filterTag		= "barValue ~= 0",
	fightIndex		= 0,
	text			= { { }, { tag= "[isUsingForeignThreatSource:\"*\"][barValue:short]" } },
	description		= L["Threat"],
}

Assessment.DefaultDisplaySets.TPS =
{
	titleTag		= "TPS: [combatTime:formatDuration]",
	listType		= "ThreatList", 
	barValueTag		= "threat / maximum(combatTime, 1)", 
	filterTag		= "barValue ~= 0",
	fightIndex		= 1,
	text			= { { }, { tag= "[isUsingForeignThreatSource:\"*\"][barValue:format(\"%.1f\")]" } },
	description		= L["TPS"],
	favorite		= false,
}

-----------------------------------------------------------------------------

Assessment.DefaultTags.ThreatDeficit =
{
	tag			= "[(maxTargetThreat-targetThreat):short]",
	fightIndex	= 0,
	description	= L["Threat Deficit"]
}

Assessment.DefaultTags.TargetAggroThreatDeficit =
{
	tag			= "[(aggrogaintargetthreat-targetThreat):short]",
	fightIndex	= 0,
	description	= L["Threat gap before Aggro gain on target"]
}

-----------------------------------------------------------------------------

function AssessmentThreat:OnInitialize()
end

function AssessmentThreat:OnEnable()
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "OnPlayerTargetChanged")
	self:RegisterEvent("Threat_ThreatUpdated", "OnThreatUpdated")
	self:RegisterEvent("Threat_ThreatCleared", "OnThreatCleared")
end

function AssessmentThreat:OnDisable()
end

-----------------------------------------------------------------------------

function AssessmentThreat:OnPlayerTargetChanged()
	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.target = GetTime()
	end
end

function AssessmentThreat:OnThreatCleared()
	local data = Assessment:GetFightData(1)
	if data and data.threat then
		data.threatLocked = true
	end
	
	data = Assessment:GetFightData(0)
	if data.threat then
		data.dataUpdateTimes.threat = GetTime()
		data.threat = self:ReleaseTable(data.threat, true)
	end
end

function AssessmentThreat:OnThreatUpdated(playerName, playerUnit, targetName, threatValue)
	for data in self:IterateUpdateDataTables() do
		data.dataUpdateTimes.threat = GetTime()
		
		local threat = data.threat
		if not threat then
			threat = self:NewTable()
			data.threat = threat
		end
		
		local playerData = threat[playerName]
		if playerData then
			if targetName == THREAT_GLOBAL_TARGET and threatValue == 0 then
				if not data.threatLocked then
					threat[playerName] = self:ReleaseTable(playerData, true)
				end
			else
				if threatValue == 0 then
					playerData.targetDetails[targetName] = nil
				else
					playerData.targetDetails[targetName] = threatValue
				end
				
				if threatValue >= playerData.amount then
					playerData.amount = threatValue
				else
					local maximumThreatValue = threatValue
				
					for k,v in pairs(playerData.targetDetails) do
						if v > maximumThreatValue then
							maximumThreatValue = v
						end
					end
					
					if maximumThreatValue == 0 then
						threat[playerName] = self:ReleaseTable(playerData, true)
					else
						playerData.amount = maximumThreatValue
					end
				end
			end
		elseif threatValue ~= 0 then
			playerData = self:NewTable()
			playerData.amount = threatValue
			playerData.targetDetails = self:NewTable()
			playerData.targetDetails[targetName] = threatValue
			threat[playerName] = playerData
		end
	end
end

-----------------------------------------------------------------------------
